/* $Id: create_mosaic.h,v 1.6 2002/01/11 08:44:55 fors Exp $
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * COPYRIGHT (c) 2001 European Southern Observatory
 * LICENSE: GNU General Public License version 2 or later
 *
 * PROJECT:   VLT Data Flow System
 * AUTHOR:    Ralf Palsa -- ESO/DMD/DPG
 * SUBSYSTEM: Instrument pipelines
 *
 * PURPOSE:
 * DESCRIPTION:
 *
 * $Name: fsmosaic-1_0 $
 * $Revision: 1.6 $
 * ----------------------------------------------------------------------------
 */
 
#ifndef _MOSAIC_H
#define _MOSAIC_H

/*
 * FITS keyword aliases
 */

#define INSTRUMENT  "INSTRUME"
#define MJD         "MJD-OBS"

#define EXTNAME   "EXTNAME"
#define ORIGFILE  "ORIGFILE"
#define ARCFILE   "ARCFILE"
#define CHECKSUM  "CHECKSUM"

#define ROTATION  "HIERARCH ESO DET CHIP1 RGAP"
#define XSHIFT    "HIERARCH ESO DET CHIP1 XGAP"
#define YSHIFT    "HIERARCH ESO DET CHIP1 YGAP"
#define XSIZE     "HIERARCH ESO DET CHIP1 PSZX"
#define YSIZE     "HIERARCH ESO DET CHIP1 PSZY"
#define BINX      "HIERARCH ESO DET WIN1 BINX"
#define BINY      "HIERARCH ESO DET WIN1 BINY"

#define N_CHIPS      "HIERARCH ESO DET CHIPS"
#define CHIP1_ID     "HIERARCH ESO DET CHIP1 ID"
#define CHIP1_NAME   "HIERARCH ESO DET CHIP1 NAME"
#define CHIP1_DATE   "HIERARCH ESO DET CHIP1 DATE"
#define CHIP1_INDEX  "HIERARCH ESO DET CHIP1 INDEX"
#define CHIP1_X      "HIERARCH ESO DET CHIP1 X"
#define CHIP1_Y      "HIERARCH ESO DET CHIP1 Y"
#define CHIP1_NX     "HIERARCH ESO DET CHIP1 NX"
#define CHIP1_NY     "HIERARCH ESO DET CHIP1 NY"
#define CHIP1_PSZX   "HIERARCH ESO DET CHIP1 PSZX"
#define CHIP1_PSZY   "HIERARCH ESO DET CHIP1 PSZY"

#define CHIP2_ID     "HIERARCH ESO DET CHIP2 ID"
#define CHIP2_NAME   "HIERARCH ESO DET CHIP2 NAME"
#define CHIP2_DATE   "HIERARCH ESO DET CHIP2 DATE"
#define CHIP2_INDEX  "HIERARCH ESO DET CHIP2 INDEX"
#define CHIP2_X      "HIERARCH ESO DET CHIP2 X"
#define CHIP2_Y      "HIERARCH ESO DET CHIP2 Y"
#define CHIP2_NX     "HIERARCH ESO DET CHIP2 NX"
#define CHIP2_NY     "HIERARCH ESO DET CHIP2 NY"
#define CHIP2_PSZX   "HIERARCH ESO DET CHIP2 PSZX"
#define CHIP2_PSZY   "HIERARCH ESO DET CHIP2 PSZY"

#define N_OUTPUTS      "HIERARCH ESO DET OUTPUTS"
#define OUTPUT1_ID     "HIERARCH ESO DET OUT1 ID"
#define OUTPUT1_NAME   "HIERARCH ESO DET OUT1 NAME"
#define OUTPUT1_INDEX  "HIERARCH ESO DET OUT1 INDEX"
#define OUTPUT1_CHIP   "HIERARCH ESO DET OUT1 CHIP"
#define OUTPUT1_X      "HIERARCH ESO DET OUT1 X"
#define OUTPUT1_Y      "HIERARCH ESO DET OUT1 Y"
#define OUTPUT1_NX     "HIERARCH ESO DET OUT1 NX"
#define OUTPUT1_NY     "HIERARCH ESO DET OUT1 NY"
#define OUTPUT1_PRSCX  "HIERARCH ESO DET OUT1 PRSCX"
#define OUTPUT1_OVSCX  "HIERARCH ESO DET OUT1 OVSCX"
#define OUTPUT1_PRSCY  "HIERARCH ESO DET OUT1 PRSCY"
#define OUTPUT1_OVSCY  "HIERARCH ESO DET OUT1 OVSCY"
#define OUTPUT1_CONAD  "HIERARCH ESO DET OUT1 CONAD"
#define OUTPUT1_GAIN   "HIERARCH ESO DET OUT1 GAIN"
#define OUTPUT1_RON    "HIERARCH ESO DET OUT1 RON"

#define OUTPUT2_ID     "HIERARCH ESO DET OUT2 ID"
#define OUTPUT2_NAME   "HIERARCH ESO DET OUT2 NAME"
#define OUTPUT2_INDEX  "HIERARCH ESO DET OUT2 INDEX"
#define OUTPUT2_CHIP   "HIERARCH ESO DET OUT2 CHIP"
#define OUTPUT2_X      "HIERARCH ESO DET OUT2 X"
#define OUTPUT2_Y      "HIERARCH ESO DET OUT2 Y"
#define OUTPUT2_NX     "HIERARCH ESO DET OUT2 NX"
#define OUTPUT2_NY     "HIERARCH ESO DET OUT2 NY"
#define OUTPUT2_PRSCX  "HIERARCH ESO DET OUT2 PRSCX"
#define OUTPUT2_OVSCX  "HIERARCH ESO DET OUT2 OVSCX"
#define OUTPUT2_PRSCY  "HIERARCH ESO DET OUT2 PRSCY"
#define OUTPUT2_OVSCY  "HIERARCH ESO DET OUT2 OVSCY"
#define OUTPUT2_CONAD  "HIERARCH ESO DET OUT2 CONAD"
#define OUTPUT2_GAIN   "HIERARCH ESO DET OUT2 GAIN"
#define OUTPUT2_RON    "HIERARCH ESO DET OUT2 RON"

#define N_WINDOWS     "HIERARCH ESO DET WINDOWS"
#define WINDOW1_STRX  "HIERARCH ESO DET WIN1 STRX"
#define WINDOW1_STRY  "HIERARCH ESO DET WIN1 STRY"
#define WINDOW1_NX    "HIERARCH ESO DET WIN1 NX"
#define WINDOW1_NY    "HIERARCH ESO DET WIN1 NY"

#define DPR_CATEGORY   "HIERARCH ESO DPR CATG"
#define DPR_TYPE       "HIERARCH ESO DPR TYPE"
#define DPR_TECHNIQUE  "HIERARCH ESO DPR TECH"

#define PRO_CATEGORY  "HIERARCH ESO PRO CATG"


/*
 * Error codes
 */

#define EXIT_IS_NO_FILE           2
#define EXIT_NO_PERMISSION        3
#define EXIT_IS_NO_FITS_FILE      4
#define EXIT_SAVE_ERROR           5
#define EXIT_IS_MOSAIC            6
#define EXIT_SAME_FILE            7
#define EXIT_FITS_ERROR           8
#define EXIT_MISSING_KEYWORD      9
#define EXIT_NO_SLAVE_FOUND      10
#define EXIT_NO_MASTER_FOUND     11
#define EXIT_UNKNOWN_CCD_ID      12
#define EXIT_WRONG_KEY_VALUE     13
#define EXIT_ALIGN_ERROR         14
#define EXIT_NO_INSTRUMENT       15
#define EXIT_UNKNOWN_INSTRUMENT  16


/*
 * Chip identifier for master and slave
 */

#define CHIP_MASTER  "CCID20-14-5-3"
#define CHIP_SLAVE   "CCID20-14-5-6"

#define INSTRUMENT_ID  "FORS2"
#define MOSAIC_ID      "FORS2_MERGED_IMAGE"

#define TINY  (1.e-8)

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
#define __BEGIN_DECLS extern "C" {
#define __END_DECLS }
#else
#define __BEGIN_DECLS
#define __END_DECLS
#endif
 
__BEGIN_DECLS

int is_mosaic(const char *);
int create_mosaic(const char *, const char *);

__END_DECLS
#endif

