/* $Id: qfits_ext.h,v 1.5 2002/01/15 10:18:38 fors Exp $
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * COPYRIGHT (c) 2001 European Southern Observatory
 * LICENSE: GNU General Public License version 2 or later
 *
 * PROJECT:   VLT Data Flow System
 * AUTHOR:    Ralf Palsa -- ESO/DMD/DPG
 * SUBSYSTEM: Instrument pipelines
 *
 * PURPOSE:
 * DESCRIPTION:
 *
 * $Name: fsmosaic-1_0 $
 * $Revision: 1.5 $
 * ----------------------------------------------------------------------------
 */
 
#ifndef _FITS_H
#define _FITS_H

#include <qfits.h>
#include <image.h>

#define FITS_CARD_SIZE  (80)
#define BSCALE_MIN  (1.0e-50)

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
#define __BEGIN_DECLS extern "C" {
#define __END_DECLS }
#else
#define __BEGIN_DECLS
#define __END_DECLS
#endif
 
__BEGIN_DECLS

struct _fits_info_t_ {
    int naxes;
    int *naxis;
    int bits_per_pixel;
    size_t header_size;
    double bscale;
    double bzero;
    int time_flag;
};

typedef struct _fits_info_t_ fits_info_t;

fits_info_t *fits_info_new(void);
void fits_info_del(fits_info_t*);

fits_info_t *fits_get_file_info(const char *);

pixel_t *fits_load_pixel_data(const char *, fits_info_t *);

image_t *fits_load_image(const char *);
int fits_save_image(const char *, qfits_header *, image_t *, int);

void fits_header_set_value(qfits_header *, const char *, const char *);

__END_DECLS
#endif
/* end of qfits_ext.h */
