# QFITS_PROG_CC_FLAG(FLAG, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#-------------------------------------------------------------------
AC_DEFUN(QFITS_PROG_CC_FLAG,
[
    AC_REQUIRE([AC_PROG_CC])

    flag=`echo $1 | sed 'y%.=/+-%___p_%'`
    AC_CACHE_CHECK([whether $CC supports -$1],
                   [qfits_cv_prog_cc_$flag],
                   [
                       eval "qfits_cv_prog_cc_$flag=no"
                       AC_LANG_PUSH(C)

                       echo 'int main() { return 0; }' >conftest.$ac_ext

                       try_compile="`$CC -$1 -c conftest.$ac_ext 2>&1`"
                       if test -z "$try_compile"; then
                           try_link="`$CC -$1 -o conftest$ac_exeext \
                                    conftest.$ac_ext 2>&1`"
                           if test -z "$try_link"; then
                               eval "qfits_cv_prog_cc_$flag=yes"
                           fi
                       fi
                       rm -f conftest*

                       AC_LANG_POP(C)
                   ])

    if eval "test \"`echo '$qfits_cv_prog_cc_'$flag`\" = yes"; then
        :
        $2
    else
        :
        $3
    fi
])


# QFITS_ENABLE_DEBUG(debug=no)
#-------------------------------
AC_DEFUN(QFITS_ENABLE_DEBUG,
[
    AC_REQUIRE([AC_PROG_CC])

    AC_ARG_ENABLE(debug,
                  AC_HELP_STRING([--enable-debug],
                                 [creates debugging code [[default=$1]]]),
                  qfits_enable_debug=$enableval, qfits_enable_debug=$1)

    AC_CACHE_CHECK([whether debugging code should be created],
                   qfits_cv_enable_debug,
                   qfits_cv_enable_debug=$qfits_enable_debug)

    if test x"$qfits_cv_enable_debug" = xyes; then

        qfits_clean_CFLAGS="`echo $CFLAGS | sed -e 's/-O[[0-9]]//g' \
                                                -e 's/-O//g'`"

        if test x"$ac_cv_prog_cc_g" = xyes; then
            qfits_clean_CFLAGS="`echo $qfits_clean_CFLAGS | sed -e 's/-g//g'`"
            CFLAGS="-g"
        else
            CFLAGS=""
        fi

        QFITS_PROG_CC_FLAG([Wall], [CFLAGS="$CFLAGS -Wall"])
        QFITS_PROG_CC_FLAG([W], [CFLAGS="$CFLAGS -W"])

        CFLAGS="$CFLAGS $qfits_clean_CFLAGS"
    else
        CFLAGS="`echo $CFLAGS | sed -e 's/-g//g'`"
    fi
])


# QFITS_ENABLE_STRICT(strict=no)
#-------------------------------
AC_DEFUN(QFITS_ENABLE_STRICT,
[
    AC_REQUIRE([AC_PROG_CC])

    AC_ARG_ENABLE(strict,
                  AC_HELP_STRING([--enable-strict],
                                 [compiles with strict compiler options (may not work! [[default=$1]]]),
                  qfits_enable_strict=$enableval, qfits_enable_strict=$1)

    AC_CACHE_CHECK([whether strict compiler options should be used],
                   qfits_cv_enable_strict,
                   qfits_cv_enable_strict=$qfits_enable_strict)

 
    if test x"$qfits_cv_enable_strict" = xyes; then
      QFITS_PROG_CC_FLAG([ansi], [CFLAGS="$CFLAGS -ansi"])
      QFITS_PROG_CC_FLAG([pedantic], [CFLAGS="$CFLAGS -pedantic"])
    fi
])


# QFITS_CHECK_CHAR_BIT
#---------------------
AC_DEFUN(QFITS_CHECK_CHAR_BIT,
[
    qfits_have_char_bit=no
    qfits_have_char_8_bits=no

    AC_MSG_CHECKING([for bits per char])

    AC_CACHE_VAL(qfits_cv_have_char_8_bits,
    [
        AC_LANG_PUSH(C)
        
        qfits_cppflags_save="$CPPFLAGS"
        qfits_cflags_save="$CFLAGS"
        qfits_ld_flags_save="$LDFLAGS"
        qfits_libs_save="$LIBS"
  
        if test "$GCC" = "yes"; then
            CFLAGS="$CFLAGS -pedantic-errors"
            CPPFLAGS="$CPPFLAGS $CFLAGS"
        fi

        AC_TRY_COMPILE([#include <limits.h>],
                       [int i = CHAR_BIT;],
                       qfits_have_char_bit=yes, qfits_have_char_bit=no)

        if test "x$qfits_have_char_bit" = "xyes"; then
            AC_TRY_RUN(
            [
                #include <limits.h>

                int main() {

                  if (CHAR_BIT != 8)
                    return 1;

                  return 0;
                }
            ], qfits_have_char_8_bits=yes, qfits_have_char_8_bits=no)
        else
            AC_TRY_RUN(
            [
                #include <limits.h>

                int main() {

                  char c = 1;
                  int i = 0;

                  while (c) {
                    c <<= 1;
                    i++;
                  }

                  if (i != 8)
                    return 1;

                  return 0;
                }
            ], qfits_have_char_8_bits=yes, qfits_have_char_8_bits=no)
        fi

        CPPFLAGS="$qfits_cppflags_save"
        CFLAGS="$qfits_cflags_save"
        LDFLAGS="$qfits_ldflags_save"
        LIBS="$qfits_libs_save"

        AC_LANG_POP

        qfits_cv_have_char_8_bits="qfits_have_char_bit=$qfits_have_char_bit \
        qfits_have_char_8_bits=$qfits_have_char_8_bits"

    ])

    eval "$qfits_cv_have_char_8_bits"
    
    if test "$qfits_have_char_8_bits" = "no"; then
        AC_MSG_ERROR([C type char is not 8 bits wide!])
    else
        if test "$qfits_have_char_bit" = "no"; then
            AC_DEFINE(CHAR_BIT, 8, [Width of type 'char'])
        else
            AC_DEFINE(HAVE_CHAR_BIT, 1,
                      [Define if CHAR_BIT is defined in limits.h and equals 8])
        fi
        AC_MSG_RESULT([8])
    fi
])
